import Raven from 'raven-js';
import axios from 'axios';

const baseURL = 'https://stumpf.kitware.com/api/mturk/v1';
const devData = {
  assignment: {
    task: {
      video: {
        path: 'https://diva-mturk-video-catalog.s3.amazonaws.com/encoded/2018-03-15_10-40-09_10-45-09_G339_school.webm',
      },
      args: {
        start: 40,
        end: 60,
      },
    },
  },
};

function captureResponse(resp) {
  try {
    Raven.captureBreadcrumb({
      message: `${resp.config.method.toUpperCase()} ${resp.config.url} => ${resp.status}`,
      category: 'axios response',
      level: 'info',
      data: resp.data,
    });
  } catch (err) {
    Raven.captureException(err);
  }
  return resp;
}

function captureRequest(config) {
  try {
    Raven.captureBreadcrumb({
      message: `${config.method.toUpperCase()} ${config.url}`,
      category: 'axios request',
      level: 'info',
      data: config.data,
    });
  } catch (err) {
    Raven.captureException(err);
  }
  return config;
}

const instance = axios.create({
  baseURL,
});
instance.interceptors.request.use(captureRequest, captureRequest);
instance.interceptors.response.use(captureResponse, captureResponse);

instance.$login = async function login(params) {
  if (process.env.NODE_ENV === 'development' || params.development) {
    // add a fake delay
    await new Promise((resolve) => {
      setTimeout(resolve, 1);
    });
    this.ready = true;
    return devData;
  }
  const authParams = {
    mturk_assignment_id: params.assignmentId,
    mturk_hit_id: params.hitId,
    mturk_worker_id: params.workerId,
    app_version: process.env.VUE_APP_GITHASH,
  };
  Raven.setExtraContext(authParams);
  const resp = await instance.post('/authenticate', authParams);
  const assignment = resp.data.assignment.id;
  instance.interceptors.request.use((config) => {
    const headers = Object.assign({
      'X-Stumpf-Mturk-Token': resp.data.token,
    }, config.headers);
    const data = Object.assign({
      assignment_id: assignment,
    }, config.data);
    return Object.assign({}, config, {
      data,
      headers,
    });
  });
  Raven.setExtraContext({
    assignment,
  });
  this.ready = true;

  return resp.data;
};

export default instance;
