from flask import request


def filter_query_string(model, field, query=None):
    """
    Return a filtered query from the provided model based on a string field.

    :param model: The sqlachemy model definition
    :param query: The query to sort (defaults to model.query)
    """
    search_value = request.args.get(field, type=str)

    query = query or model.query

    if search_value is not None:
        query = query.filter(model.searchable_fields[field].contains(search_value))

    return query


def filter_query_int(model, field, query=None):
    """
    Return a filtered query from the provided model based on an int field.

    :param model: The sqlachemy model definition
    :param query: The query to sort (defaults to model.query)
    """
    search_value = request.args.get(field, type=int)

    query = query or model.query

    if search_value is not None:
        query = query.filter(model.searchable_fields[field] == search_value)

    return query


def filter_query_enum(model, field, query=None):
    """
    Return a filtered query from the provided model based on an enum field.

    :param model: The sqlachemy model definition
    :param query: The query to sort (defaults to model.query)
    """
    search_value = request.args.get(field, type=str)

    query = query or model.query

    if search_value is not None and search_value != '':
        search_value = search_value.upper()
        query = query.filter(model.searchable_fields[field] == search_value)

    return query
