import json

# TODO: make a chibitama from this module
# from stumpf.tama import ChibiTama, listen


task_query = """
SELECT answer from answer
WHERE assignment_id in (
    SELECT id
    FROM assignment
    WHERE task_id = '{task_id}'
)
"""

# at least 3 / 5 or (2 / 3) agreement required
THRESHOLD = 0.66


def process_task(engine, task_id):
    # initialize activity detection count object
    #   activity name ->
    #      [ <# of 0 detections>, <# of 1 detections>, <# of 2+ detections> ]
    activities = {}
    for row in engine.execute(task_query.format(task_id=task_id)):
        for activity, value in row[0].items():
            if activity not in activities:
                activities[activity] = [0, 0, 0]
            if value is not None:
                activities[activity][value] += 1

    # adjudicate answers (contains only activities with agreement)
    #   activity -> 0, 1, or 2
    adjudicated = {}
    for activity, counts in activities.items():
        total = sum(counts)
        if not total:
            continue
        for value, count in enumerate(counts):
            if count / total >= THRESHOLD:
                adjudicated[activity] = value
                break

    # post answer back to the task
    engine.execute(f"UPDATE task SET answer = '{json.dumps(adjudicated)}' WHERE id = '{task_id}'")


def process_all_tasks(engine, app_id):
    tasks = engine.execute(f"SELECT id FROM task WHERE annotation_app_id = '{app_id}'")
    count = 0
    for task in tasks:
        count += 1
        process_task(engine, task['id'])
    return count


if __name__ == '__main__':
    import os
    from sqlalchemy import create_engine

    uri = os.environ['STUMPF_POSTGRES_URI']
    engine = create_engine(uri)
    count = process_all_tasks(engine, '2d755948-af16-40bf-846e-0ce885a43b53')
    print(f'Processed {count} tasks')
